@@ -51,12 +51,13 @@ test_that("factor", {
   dl <- prepData(phyActFlowPropensity)
   dl$scale <- rep(1.5, dl$NITEMS)
   dl$alpha <- rnorm(dl$NITEMS, 1, .1)
+  dl <- prepSingleFactorModel(dl, .2)
   m2 <- findModel("factor_ll")
   f2 <- sampling(m2, dl, chains=1, cores=0, iter=1, seed=1,warmup=0, refresh=0)
-  expect_equal(get_logposterior(f2)[[1]], -100185.1619, tolerance=1e-2, scale=1)
+  expect_equal(get_logposterior(f2)[[1]], -110344.939, tolerance=1e-2, scale=1)
   #cat(deparse(round(fivenum(extract(f2)$log_lik[1,]), 3)))
   expect_equal(fivenum(extract(f2)$log_lik[1,]),
-               c(-65.644, -8.36, -4.612, -2.054, 0), tolerance=1e-2, scale=1)
+               c(-54.219, -9.448, -4.866, -2.03, 0), tolerance=1e-2, scale=1)
 })
 
 test_that("mixed thresholds", {
@@ -96,13 +97,13 @@ test_that("mixed thresholds", {
 
 test_that("calibrateItems", {
   pafp <- phyActFlowPropensity[,c(1,2,6:8)]
-  result <- calibrateItems(pafp, iter=1000L, seed=1)
+  result <- calibrateItems(pafp, iter=1000L, chains=2, seed=1)
   expect_equal(nrow(result), 3)
-  expect_true(all(result[1:2,'n_eff'] > 400))
+  expect_true(all(result[1:2,'n_eff'] > 200))
   expect_true(all(result[1:2,'Rhat'] < 1.015))
   expect_true(result[3,'low_bfmi'] > 0 ||
                 result[3,'divergent'] > 0)
-  expect_true(result[3,'n_eff'] < 400)
+  expect_true(result[3,'n_eff'] < 200)
   expect_equal(result[1:2,'scale'], c(.469, .535),
                tolerance=.01, scale=1)
 })
